@@ -33,7 +33,6 @@ from gns3server.builtins.interfaces import get_windows_interfaces
 from .hypervisor import Hypervisor
 from .hypervisor_manager import HypervisorManager
 from .dynamips_error import DynamipsError
-from ..attic import has_privileged_access
 
 # Nodes
 from .nodes.router import Router
@@ -138,6 +137,7 @@ class Dynamips(IModule):
         self._tempdir = kwargs["temp_dir"]
         self._working_dir = self._projects_dir
         self._host = dynamips_config.get("host", kwargs["host"])
+        self._console_host = dynamips_config.get("console_host", kwargs["console_host"])
 
         if not sys.platform.startswith("win32"):
             #FIXME: pickle issues Windows
@@ -154,12 +154,19 @@ class Dynamips(IModule):
         if not sys.platform.startswith("win32"):
             self._callback.stop()
 
+        # automatically save configs for all router instances
+        for router_id in self._routers:
+            router = self._routers[router_id]
+            try:
+                router.save_configs()
+            except DynamipsError:
+                continue
+
         # stop all Dynamips hypervisors
         if self._hypervisor_manager:
             self._hypervisor_manager.stop_all_hypervisors()
 
         self.delete_dynamips_files()
-
         IModule.stop(self, signum)  # this will stop the I/O loop
 
     def _check_hypervisors(self):
@@ -225,6 +232,14 @@ class Dynamips(IModule):
         :param request: JSON request (not used)
         """
 
+        # automatically save configs for all router instances
+        for router_id in self._routers:
+            router = self._routers[router_id]
+            try:
+                router.save_configs()
+            except DynamipsError:
+                continue
+
         # stop all Dynamips hypervisors
         if self._hypervisor_manager:
             self._hypervisor_manager.stop_all_hypervisors()
@@ -254,6 +269,7 @@ class Dynamips(IModule):
         self.delete_dynamips_files()
 
         self._hypervisor_manager = None
+        self._working_dir = self._projects_dir
         log.info("dynamips module has been reset")
 
     def start_hypervisor_manager(self):
@@ -282,7 +298,7 @@ class Dynamips(IModule):
             raise DynamipsError("Cannot write to working directory {}".format(workdir))
 
         log.info("starting the hypervisor manager with Dynamips working directory set to '{}'".format(workdir))
-        self._hypervisor_manager = HypervisorManager(self._dynamips, workdir, self._host)
+        self._hypervisor_manager = HypervisorManager(self._dynamips, workdir, self._host, self._console_host)
 
         for name, value in self._hypervisor_manager_settings.items():
             if hasattr(self._hypervisor_manager, name) and getattr(self._hypervisor_manager, name) != value:
@@ -293,10 +309,8 @@ class Dynamips(IModule):
         """
         Set or update settings.
 
-        Mandatory request parameters:
-        - path (path to the Dynamips executable)
-
         Optional request parameters:
+        - path (path to the Dynamips executable)
         - working_dir (path to a working directory)
         - project_name
 
@@ -311,7 +325,9 @@ class Dynamips(IModule):
 
         #TODO: JSON schema validation
         if not self._hypervisor_manager:
-            self._dynamips = request.pop("path")
+
+            if "path" in request:
+                self._dynamips = request.pop("path")
 
             if "working_dir" in request:
                 self._working_dir = request.pop("working_dir")
@@ -347,8 +363,13 @@ class Dynamips(IModule):
                 # for local server
                 new_working_dir = request.pop("working_dir")
 
+            try:
+                self._hypervisor_manager.working_dir = new_working_dir
+            except DynamipsError as e:
+                log.error("could not change working directory: {}".format(e))
+                return
+
             self._working_dir = new_working_dir
-            self._hypervisor_manager.working_dir = new_working_dir
 
             # apply settings to the hypervisor manager
             for name, value in request.items():
