@@ -17,8 +17,10 @@
 
 import os
 import base64
+import ntpath
 import time
 from gns3server.modules import IModule
+from gns3dms.cloud.rackspace_ctrl import get_provider
 from ..dynamips_error import DynamipsError
 
 from ..nodes.c1700 import C1700
@@ -140,12 +142,25 @@ class VM(object):
         chassis = request.get("chassis")
         router_id = request.get("router_id")
 
+        # Locate the image
         updated_image_path = os.path.join(self.images_directory, image)
         if os.path.isfile(updated_image_path):
             image = updated_image_path
+        else:
+            if not os.path.exists(self.images_directory):
+                os.mkdir(self.images_directory)
+            cloud_path = request.get("cloud_path", None)
+            if cloud_path is not None:
+                # Download the image from cloud files
+                _, filename = ntpath.split(image)
+                src = '{}/{}'.format(cloud_path, filename)
+                provider = get_provider(self._cloud_settings)
+                log.debug("Downloading file from {} to {}...".format(src, updated_image_path))
+                provider.download_file(src, updated_image_path)
+                log.debug("Download of {} complete.".format(src))
+                image = updated_image_path
 
         try:
-
             if platform not in PLATFORMS:
                 raise DynamipsError("Unknown router platform: {}".format(platform))
 
@@ -597,31 +612,7 @@ class VM(object):
             return
 
         try:
-            if router.startup_config or router.private_config:
-
-                startup_config_base64, private_config_base64 = router.extract_config()
-                if startup_config_base64:
-                    try:
-                        config = base64.decodestring(startup_config_base64.encode("utf-8")).decode("utf-8")
-                        config = "!\n" + config.replace("\r", "")
-                        config_path = os.path.join(router.hypervisor.working_dir, router.startup_config)
-                        with open(config_path, "w") as f:
-                            log.info("saving startup-config to {}".format(router.startup_config))
-                            f.write(config)
-                    except OSError as e:
-                        raise DynamipsError("Could not save the startup configuration {}: {}".format(config_path, e))
-
-                if private_config_base64:
-                    try:
-                        config = base64.decodestring(private_config_base64.encode("utf-8")).decode("utf-8")
-                        config = "!\n" + config.replace("\r", "")
-                        config_path = os.path.join(router.hypervisor.working_dir, router.private_config)
-                        with open(config_path, "w") as f:
-                            log.info("saving private-config to {}".format(router.private_config))
-                            f.write(config)
-                    except OSError as e:
-                        raise DynamipsError("Could not save the private configuration {}: {}".format(config_path, e))
-
+            router.save_configs()
         except DynamipsError as e:
             log.warn("could not save config to {}: {}".format(router.startup_config, e))
 
